@@ -1,24 +1,44 @@
 #![doc(hidden)]
 
-#[cfg(feature = "plotters")]
-#[allow(unused_imports)]
+#[cfg(test)]
+use crate::*;
+
+#[test]
+fn anti_symmetry_extension() {
+    let ans = synthesis::anti_sym_ext(OPEN_CURVE1);
+    assert_eq!(ans, OPEN_CURVE1_ANS);
+}
+
+#[cfg(all(test, feature = "plot_export"))]
+fn planar_synthesis(target: &[[f64; 2]], gen: u64, pop_num: usize, open: bool) {
+    use crate::synthesis::*;
+    use indicatif::ProgressBar;
+    use plotly::ImageFormat;
+    use std::{f64::consts::TAU, fs::write};
+
+    let pb = ProgressBar::new(gen);
+    let s = mh::Solver::build(mh::De::default())
+        .task(|ctx| ctx.gen == gen)
+        .callback(|ctx| pb.set_position(ctx.gen))
+        .pop_num(pop_num)
+        .record(|ctx| ctx.best_f)
+        .solve(Planar::new(target, 720, None, open));
+    pb.finish();
+    let plot = plot::plot_history(s.report(), s.best_fitness());
+    plot.save("history.svg", ImageFormat::SVG, 800, 600, 1.);
+    let ans = s.result();
+    write("result.ron", ron::to_string(&ans).unwrap()).unwrap();
+    let curve = Mechanism::four_bar(&ans).four_bar_loop(0., TAU, 360);
+    println!("harmonic: {}", s.func().harmonic());
+    println!("seed: {}", s.seed());
+    let curves = [("Target", target), ("Optimized", &curve)];
+    let plot = plot::plot_curve("Synthesis Test", &curves);
+    plot.save("result.svg", ImageFormat::SVG, 800, 800, 1.);
+}
+
+#[cfg(feature = "plot_export")]
 #[test]
 fn planar() {
-    use crate::{
-        plot::{plot_curve, plot_history},
-        synthesis::{
-            mh::{De, Solver},
-            Planar,
-        },
-        Mechanism,
-    };
-    use indicatif::ProgressBar;
-    use ron::{from_str, to_string};
-    use std::{
-        f64::consts::TAU,
-        fs::{read_to_string, write},
-    };
-
     // let target = Mechanism::four_bar(FourBar {
     //     p0: (0., 0.),
     //     a: 0.,
@@ -53,27 +73,11 @@ fn planar() {
     // let target = YU1;
     // let target = HAND;
     // let target = OPEN_CURVE1;
+    // let target = OPEN_CURVE2;
     // let target = TRIANGLE2;
     let target = CRUNODE;
     // let target = LINE;
-    let gen = 40;
-    let pb = ProgressBar::new(gen);
-    let s = Solver::build(De::default())
-        .task(|ctx| ctx.gen == gen)
-        .callback(|ctx| pb.set_position(ctx.gen))
-        .pop_num(200)
-        .record(|ctx| ctx.best_f)
-        .solve(Planar::new(target, 720, 360));
-    pb.finish();
-    plot_history(s.report(), "history.svg");
-    let ans = s.result();
-    write("result.ron", to_string(&ans).unwrap()).unwrap();
-    let path = Mechanism::four_bar(&ans).four_bar_loop(0., 360);
-    plot_curve(
-        "Synthesis Test",
-        &[("Target", &target), ("Optimized", &path)],
-        "result.svg",
-    );
+    planar_synthesis(target, 50, 400, false);
 }
 
 pub const HAND: &[[f64; 2]] = &[
@@ -495,6 +499,269 @@ pub const OPEN_CURVE1: &[[f64; 2]] = &[
     [67.41322314049587, 25.384615384615383],
     [60.6439923712651, 18.153846153846153],
 ];
+#[cfg(test)]
+const OPEN_CURVE1_ANS: &[[f64; 2]] = &[
+    [0.0, 0.0],
+    [0.09230769230769287, 8.584615384615379],
+    [2.6461538461538456, 15.784615384615385],
+    [6.430769230769233, 20.21538461538461],
+    [16.830769230769235, 23.56923076923077],
+    [19.846153846153847, 24.15384615384615],
+    [24.246153846153845, 21.815384615384616],
+    [22.953846153846158, 19.16923076923077],
+    [20.276923076923083, 12.984615384615381],
+    [12.830769230769235, 4.338461538461537],
+    [0.0, 0.0],
+    [-12.830769230769235, -4.338461538461537],
+    [-20.276923076923083, -12.984615384615381],
+    [-22.953846153846158, -19.16923076923077],
+    [-24.246153846153845, -21.815384615384616],
+    [-19.846153846153847, -24.15384615384615],
+    [-16.830769230769235, -23.56923076923077],
+    [-6.430769230769233, -20.21538461538461],
+    [-2.6461538461538456, -15.784615384615385],
+    [-0.09230769230769287, -8.584615384615379],
+];
+pub const OPEN_CURVE2: &[[f64; 2]] = &[
+    [-1.9333001636959, -3.6211521920669134],
+    [-1.900066416633054, -3.646310302888677],
+    [-1.8750454732179942, -3.6620524988942016],
+    [-1.852115789220855, -3.6750779938143316],
+    [-1.830247674004803, -3.6865276250484937],
+    [-1.8090321440133719, -3.6968606775034805],
+    [-1.7882595405854413, -3.706316637071842],
+    [-1.7678069512660473, -3.7150389791355805],
+    [-1.7475961178888069, -3.7231216006104844],
+    [-1.727574417508887, -3.730629865474372],
+    [-1.7077051256870286, -3.737611411766564],
+    [-1.6879619650389652, -3.7441022203421515],
+    [-1.6683258392736748, -3.7501302612627505],
+    [-1.6487827697189879, -3.755717804153877],
+    [-1.6293225343699627, -3.7608829461400486],
+    [-1.6099377378522994, -3.7656406587008373],
+    [-1.5906231565970266, -3.7700035265369527],
+    [-1.571375265877268, -3.7739822824217266],
+    [-1.5521918905774408, -3.7775862029113454],
+    [-1.5330719422985262, -3.7808234067301263],
+    [-1.5140152180511666, -3.7837010835581815],
+    [-1.4950222437473792, -3.786225672068417],
+    [-1.4760941508484855, -3.7884030003076927],
+    [-1.4572325779366078, -3.790238397699899],
+    [-1.438439591285282, -3.7917367853604844],
+    [-1.419717620097883, -3.7929027496225522],
+    [-1.401069403201611, -3.79374060241568],
+    [-1.3824979447834103, -3.7942544312385693],
+    [-1.3640064773325127, -3.7944481408138335],
+    [-1.3455984303786586, -3.7943254880333326],
+    [-1.3272774039302955, -3.793890111445399],
+    [-1.309047145753865, -3.7931455562665346],
+    [-1.2909115318151088, -3.7920952956957796],
+    [-1.2728745493411047, -3.790742749152982],
+    [-1.2549402820683606, -3.7890912979405456],
+    [-1.2371128973254253, -3.787144298733151],
+    [-1.2193966346638254, -3.7849050952250356],
+    [-1.2017957958029133, -3.782377028204933],
+    [-1.1843147356954429, -3.7795634442812656],
+    [-1.1669578545538273, -3.7764677034418734],
+    [-1.1497295907037255, -3.773093185601621],
+    [-1.132634414153326, -3.7694432962659197],
+    [-1.115676820784367, -3.7655214714175447],
+    [-1.0988613270854408, -3.761331181717032],
+    [-1.082192465360083, -3.7568759360928254],
+    [-1.065674779351982, -3.7521592847855585],
+    [-1.049312820237868, -3.7471848219009978],
+    [-1.033111142945399, -3.741956187517904],
+    [-1.017074302759068, -3.736477069390052],
+    [-1.001206852181856, -3.7307512042757125],
+    [-0.9855133380243462, -3.7247823789228374],
+    [-0.9699982986962823, -3.718574430733886],
+    [-0.9546662616783447, -3.7121312481304756],
+    [-0.9395217411541961, -3.7054567706348807],
+    [-0.9245692357847454, -3.6985549886826363],
+    [-0.9098132266081793, -3.6914299431781186],
+    [-0.8952581750505513, -3.6840857248029195],
+    [-0.8809085210327812, -3.676526473085062],
+    [-0.8667686811607049, -3.668756375235567],
+    [-0.8528430469854746, -3.6607796647575723],
+    [-0.8391359833220315, -3.6526006198321013],
+    [-0.8256518266137196, -3.6442235614836394],
+    [-0.812394883331282, -3.635652851527925],
+    [-0.799369428394549, -3.6268928903037447],
+    [-0.7865797036051312, -3.6179481141900895],
+    [-0.7740299160782846, -3.6088229929097264],
+    [-0.7617242366619733, -3.599522026620075],
+    [-0.7496667983308702, -3.5900497427922957],
+    [-0.7378616945427905, -3.5804106928796333],
+    [-0.7263129775446864, -3.5706094487763598],
+    [-0.7150246566150034, -3.560650599069129],
+    [-0.7040006962288339, -3.550538745083175],
+    [-0.693245014131936, -3.540278496726572],
+    [-0.6827614793093727, -3.5298744681367578],
+    [-0.6725539098342053, -3.5193312731346476],
+    [-0.662626070581459, -3.5086535204930196],
+    [-0.6529816707923852, -3.497845809027347],
+    [-0.6436243614740104, -3.486912722518993],
+    [-0.634557732618991, -3.475858824482551],
+    [-0.6257853102310036, -3.4646886527912337],
+    [-0.6173105531412739, -3.453406714176474],
+    [-0.6091368496023974, -3.4420174786203237],
+    [-0.6012675136464232, -3.430525373661908],
+    [-0.5937057811951862, -3.4189347786418716],
+    [-0.5864548059122174, -3.4072500189116868],
+    [-0.5795176547871649, -3.3954753600376177],
+    [-0.5728973034456333, -3.3836150020321885],
+    [-0.5665966311796347, -3.3716730736490197],
+    [-0.5606184156965162, -3.359653626779913],
+    [-0.5549653275872883, -3.347560630995992],
+    [-0.5496399245186969, -3.3353979682774284],
+    [-0.5446446451572153, -3.3231694279789066],
+    [-0.5399818028373247, -3.3108787020801382],
+    [-0.5356535789910093, -3.2985293807727056],
+    [-0.5316620163602725, -3.286124948435859],
+    [-0.5280090120196569, -3.2736687800548268],
+    [-0.5246963102411476, -3.2611641381354173],
+    [-0.5217254952394121, -3.248614170168231],
+    [-0.5190979838409717, -3.236021906694572],
+    [-0.5168150181265465, -3.223390260023997],
+    [-0.5148776581013244, -3.2107220236504372],
+    [-0.5132867744531902, -3.1980198724097986],
+    [-0.5120430414638598, -3.1852863634169464],
+    [-0.5111469301422573, -3.1725239378139505],
+    [-0.5105987016532522, -3.1597349233544736],
+    [-0.5103984011178175, -3.1469215378411732],
+    [-0.5105458518627313, -3.1340858934241607],
+    [-0.511882160807287, -3.1083557800112054],
+    [-0.5130695128096245, -3.0954650576875298],
+    [-0.5146015965989189, -3.082559584254694],
+    [-0.5164770614996892, -3.0696410375053595],
+    [-0.5186943143228754, -3.056711032611443],
+    [-0.5212515188729502, -3.043771131798084],
+    [-0.5241465964564044, -3.0308228545599527],
+    [-0.5273772274310684, -3.0178676883320703],
+    [-0.5309408538248706, -3.0049070995187357],
+    [-0.5348346830408018, -2.9919425447767707],
+    [-0.5390556926522536, -2.978975482443424],
+    [-0.5436006362797356, -2.9660073839950036],
+    [-0.5484660505265389, -2.953039745419813],
+    [-0.5536482629374309, -2.940074098388352],
+    [-0.5591434009312578, -2.9271120211050254],
+    [-0.5649474016456425, -2.9141551487288195],
+    [-0.5710560226201075, -2.9012051832554895],
+    [-0.5774648532331264, -2.8882639027606727],
+    [-0.5841693267991087, -2.875333169911827],
+    [-0.5911647332233014, -2.8624149396668703],
+    [-0.5984462321062249, -2.8495112660885717],
+    [-0.6060088661846758, -2.83662430821594],
+    [-0.6138475749935927, -2.8237563349467525],
+    [-0.6219572086322043, -2.8109097288987184],
+    [-0.630332541518867, -2.798086989230274],
+    [-0.6389682860217605, -2.785290733415403],
+    [-0.6478591058570584, -2.7725236979798584],
+    [-0.656999629152146, -2.7597887382185884],
+    [-0.6663844610787962, -2.747088826925637],
+    [-0.6760081959696862, -2.7344270521783725],
+    [-0.6858654288410453, -2.7218066142271646],
+    [-0.6959507662543604, -2.70923082154971],
+    [-0.7062588364606512, -2.696703086135865],
+    [-0.7167842987816989, -2.684226918074112],
+    [-0.7275218521934887, -2.671805919514665],
+    [-0.7384662430878588, -2.659443778086692],
+    [-0.7496122721987339, -2.647144259848325],
+    [-0.7609548006891785, -2.6349112018480603],
+    [-0.7724887554047574, -2.62274850437496],
+    [-0.7842091333071508, -2.610660122972879],
+    [-0.7961110051096385, -2.5986500602908613],
+    [-0.8081895181428047, -2.5867223578380023],
+    [-0.8204398984846675, -2.574881087706672],
+    [-0.8328574523943152, -2.563130344323059],
+    [-0.8454375670921224, -2.551474236278737],
+    [-0.8581757109326779, -2.5399168782915047],
+    [-0.8710674330187844, -2.5284623833381312],
+    [-0.8841083623063046, -2.5171148549960627],
+    [-0.8972942062503009, -2.5058783800256217],
+    [-0.9106207490429297, -2.4947570212188808],
+    [-0.9240838494929628, -2.483754810536245],
+    [-0.937679438595703, -2.4728757425469197],
+    [-0.9514035168405072, -2.4621237681848887],
+    [-0.9652521513012231, -2.4515027888278014],
+    [-0.9792214725526247, -2.4410166507023323],
+    [-0.9933076714534841, -2.430669139616061],
+    [-1.0075069958343175, -2.420463976012815],
+    [-1.0218157471251081, -2.4104048103456512],
+    [-1.0362302769555414, -2.4004952187592337],
+    [-1.0507469837574803, -2.3907386990712767],
+    [-1.0653623093966562, -2.381138667040979],
+    [-1.0800727358578284, -2.3716984529108482],
+    [-1.094874782005059, -2.3624212982071455],
+    [-1.109765000436231, -2.3533103527831467],
+    [-1.1247399744485678, -2.3443686720886907],
+    [-1.1397963151296773, -2.33559921464885],
+    [-1.1549306585865602, -2.327004839734151],
+    [-1.1701396633231145, -2.3185883052044334],
+    [-1.1854200077749106, -2.310352265508246],
+    [-1.2007683880084357, -2.3022992698195157],
+    [-1.2161815155905877, -2.2944317602931505],
+    [-1.2316561156329557, -2.2867520704211532],
+    [-1.2471889250143426, -2.2792624234707604],
+    [-1.2627766907840634, -2.271964930986033],
+    [-1.2784161687477833, -2.2648615913341485],
+    [-1.2941041222370628, -2.257954288277478],
+    [-1.3098373210632992, -2.2512447895521537],
+    [-1.3256125406564534, -2.244734745433441],
+    [-1.3414265613887792, -2.2384256872676067],
+    [-1.3572761680837417, -2.2323190259491943],
+    [-1.3731581497104393, -2.226416050321631],
+    [-1.3890692992641096, -2.2207179254778278],
+    [-1.4050064138337155, -2.215225690935864],
+    [-1.4209662948581923, -2.209940258662955],
+    [-1.436945748573687, -2.204862410918556],
+    [-1.4529415866550512, -2.1999927978846294],
+    [-1.4689506270559998, -2.1953319350477343],
+    [-1.4849696950537117, -2.190880200293492],
+    [-1.5009956245052936, -2.1866378306691017],
+    [-1.5170252593254543, -2.182604918763747],
+    [-1.533055455197052, -2.1787814086496913],
+    [-1.549083081528883, -2.175167091318493],
+    [-1.5651050236783064, -2.171761599536648],
+    [-1.5811181854601342, -2.1685644020328576],
+    [-1.597119491967781, -2.1655747969144388],
+    [-1.6131058927381354, -2.1627919041926895],
+    [-1.629074365298203, -2.1602146572754246],
+    [-1.6450219191395177, -2.1578417932586427],
+    [-1.6609456001760132, -2.1556718418169867],
+    [-1.676842495752851, -2.1537031124529618],
+    [-1.6927097402883025, -2.1519336798156075],
+    [-1.708544521648803, -2.150361366737965],
+    [-1.7243440883798116, -2.148983724565725],
+    [-1.7401057579434065, -2.147798010252197],
+    [-1.7558269261493482, -2.146801159571121],
+    [-1.7715050780120847, -2.1459897556402656],
+    [-1.7871378003250578, -2.1453599917437645],
+    [-1.8027227963202068, -2.144907627173668],
+    [-1.8182579028810515, -2.1446279344588],
+    [-1.8337411109110286, -2.144515635879849],
+    [-1.8491705896376498, -2.144564826537877],
+    [-1.8645447158760886, -2.1447688803822014],
+    [-1.879862109610511, -2.1451203344137095],
+    [-1.8951216777191666, -2.145610744611503],
+    [-1.9103226683334107, -2.1462305047541888],
+    [-1.9254647392808057, -2.146968615861071],
+    [-1.9405480454773527, -2.1478123888828575],
+    [-1.9555733522653318, -2.1487470555707766],
+    [-1.9705421849833553, -2.14975525052777],
+    [-1.9854570302757721, -2.1508163084622787],
+    [-2.000321613196126, -2.151905289483753],
+    [-2.015141288667747, -2.1529915921811074],
+    [-2.029923611544212, -2.1540369199365133],
+    [-2.044679197264645, -2.154992190019469],
+    [-2.0594230793020616, -2.155792626839745],
+    [-2.0741769693642467, -2.156349539980583],
+    [-2.0889732900992244, -2.156535561991003],
+    [-2.103863061539208, -2.1561555978414857],
+    [-2.1189334407333624, -2.1548818068597546],
+    [-2.1343552126700542, -2.1520764136078174],
+    [-2.1505663614628374, -2.1461020265188284],
+];
 pub const TRIANGLE1: &[[f64; 2]] = &[
     [140.1207548649233, 240.19095327938354],
     [122.18597225622764, 236.93008371416616],
